% Basic model - Ramsey policy - LABOR AND CONSUMPTION TAXES (NO LUMP-SUM)
% Choosing optimal labor taxes (consumption taxes determined by government budget constraint)

//------------------------------------------------------------------------------------------------------------------------
//1. Variable declaration
//------------------------------------------------------------------------------------------------------------------------

// 15 Endogenous variables
var c n w k rk iv r d tauc taun y tb g a;

// 2 Exogenous variables 

varexo ash gsh;
 
//------------------------------------------------------------------------------------------------------------------------
// 2. Parameter declaration and calibration
//-------------------------------------------------------------------------------------------------------------------------
 
parameters GAM1 RS D A G BETTA DELTA ALFA PSI THETA A1 A2 RHOA RHOG SIGA SIGG ; 

% Calibrated parameters

BETTA = 1.04^(-1/4);
DELTA = 1.1^(1/4)-1;
ALFA = 0.32;
PSI = 0.02;
THETA = 0.101;
A1 = DELTA / (1 - THETA);
A2 = DELTA ^(1/THETA);

RHOA = 0.42;
RHOG = 0.8;
SIGA = 0.0129;
SIGG = 0.005;

D_Y  = 0.60;
G_Y  = 0.20;
TAUN = 0.42;

[GAM1, RS, D, A, G] = ss_competitive_equilibrium(0.25,TAUN,BETTA,DELTA,ALFA,D_Y,G_Y);

M_.params(1) = GAM1; 
M_.params(2) = RS;
M_.params(3) = D;
M_.params(4) = A;
M_.params(5) = G;

//-----------------------------------------------------------------------------------------------------------------------
// 3. The model
//-----------------------------------------------------------------------------------------------------------------------

model;

k - (1 - DELTA) * k(-1) - (A1 + (THETA*A2 / (THETA - 1)) * ((iv/k(-1))^(1-1/THETA)))* k(-1) = 0;            //capital accumulation (1)

1 / (c * (1 + tauc)) - BETTA * r(+1) * (1 / (c(+1) * (1 + tauc(+1)))) = 0;                                  //Euler equation (2) 

GAM1 / ( 1 - n) - (1 - taun) * w / ((1 + tauc) * c) = 0;                                                    //labor choice (3) 

((iv/k(-1))^(1/THETA))/ (c * (1 + tauc)) - BETTA * (1 / (c(+1) * (1 + tauc(+1)))) * 
(A2*rk(+1) + ((iv(+1)/k)^(1/THETA))*( 1 - DELTA + A1 + (A2/(THETA-1))* (iv(+1)/k)^(1 - 1/THETA) ) ) = 0;    //capital choice (4) 

a * ((k(-1))^ALFA) * (n^(1-ALFA)) - y = 0;                                                                  //production function (5)

w - (1 - ALFA) * a * (k(-1)/n)^ALFA = 0;                                                                    //wage = MP of labor (6)

rk - ALFA * a * (k(-1)/n)^(ALFA - 1) = 0;                                                                   //rental rate = MP of capital (7) 

taun * w * n + tauc * c - g = 0;                                                                            //government budget constraint (8)

y - ( c + iv + g + tb ) = 0;                                                                                //aggregate resource constraint (9)

d - (r * d(-1) - tb) = 0;                                                                                   //foreign debt (10)

r - ( RS + PSI * (exp(d - D) - 1)) = 0;                                                                     //nominal interest rate (11) 


a - (1-RHOA)*A - RHOA*a(-1) - SIGA*ash = 0;                                                                 // Technology shock (12)

g - (1-RHOG)*G - RHOG*g(-1) - SIGG*gsh = 0;                                                                 // Government spending shock (13) 

end; 

write_latex_dynamic_model; 

//--------------------------------------------------------------------------------------------------------------------------
// 4. Steady state
//---------------------------------------------------------------------------------------------------------------------------

initval;
taun = 0.42;
end;

steady_state_model;

a = 1;

d = D;

g = G;

r = 1 / BETTA;

rk = (1 / BETTA) - (1 - DELTA);

w = (1 - ALFA) * ((( (1/ALFA) * rk ) ^ ( 1 / (ALFA-1) ))^ALFA);

tb = (r-1) * d ;

y = (tb + (1 - taun) * w / GAM1) / (1 - (DELTA * ALFA / rk) + ((1 - ALFA) * (1 - taun) / GAM1) - (1 - ALFA) * taun);

k = ALFA * y / rk;

n = (1 - ALFA) * y / w;

c = (1 - taun) * w * (1 - n)/ GAM1 - g + taun * w * n;

tauc = (g - taun* w * n)/c;

iv = DELTA * k;

end;

//---------------------------------------------------------------------------------------------------------------------------
// 5. shocks
//---------------------------------------------------------------------------------------------------------------------------

shocks;

var ash; stderr 1;
var gsh; stderr 1;

end;

//--------------------------------------------------------------------------------------------------------------------------
// 6. Ramsey problem
//--------------------------------------------------------------------------------------------------------------------------

planner_objective( log(c)+ GAM1 * log (1-n) );

ramsey_policy(planner_discount=BETTA,order=1, instruments = (taun), nograph);
